Files
project_6/cccl_upstream/cudax/test/stf/cpp/reuse_computation.cu
muh-bot dedf08166a [CCCL] Add missing CCCL components: c2h, nvbench_helper, cmake, cudax, AGENTS.md
Added 863 files from NVIDIA/cccl sparse checkout:
- c2h/ (27 files): Catch2 test helpers — generators, validators, runner
- nvbench_helper/ (10 files): Benchmark harness utilities
- cmake/ (29 files): CMake presets and build helpers
- cudax/ (794 files): Experimental CUDA extensions
- AGENTS.md: NVIDIA's official AI agent instructions for CCCL
- CMakePresets.json: Standardized build configurations
- cccl-version.json: Version tracking

Also added CCCL_ASSET_MAP.md mapping all 4295 CCCL files to
competition value and PRD items.

cccl_upstream now covers 100% of competition-critical assets:
- 27 tuning headers (SM80/90/100 benchmark data)
- 32 dispatch headers (algorithm implementations)
- 60 Thrust examples (correctness verification)
- 217 CUB Catch2 tests (regression matrix)
- 153 CUB benchmarks (parameter space search)
- 18 CUB examples (API verification)
- 27 test helpers + benchmark harness
- 794 cudax experimental extensions
2026-08-06 02:14:18 +00:00

81 lines
2.0 KiB
Plaintext

//===----------------------------------------------------------------------===//
//
// Part of CUDASTF in CUDA C++ Core Libraries,
// under the Apache License v2.0 with LLVM Exceptions.
// See https://llvm.org/LICENSE.txt for license information.
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
// SPDX-FileCopyrightText: Copyright (c) 2022-2024 NVIDIA CORPORATION & AFFILIATES.
//
//===----------------------------------------------------------------------===//
#include <cuda/experimental/__stf/utility/run_once.cuh>
#include <cuda/experimental/stf.cuh>
using namespace cuda::experimental::stf;
int main()
{
context ctx;
const int N = 16;
size_t niter = 12;
int A[N];
for (int i = 0; i < N; i++)
{
A[i] = 2 * i + 1;
}
auto lres = ctx.logical_data(A);
for (size_t k = 0; k < niter; k++)
{
auto ltmp = ctx.logical_data(lres.shape());
ctx.parallel_for(ltmp.shape(), ltmp.write())->*[] __device__(size_t i, auto tmp) {
tmp(i) = i;
};
ctx.parallel_for(lres.shape(), ltmp.read(), lres.rw())->*[] __device__(size_t i, auto tmp, auto res) {
res(i) += tmp(i);
};
}
for (size_t k = 0; k < niter; k++)
{
auto ltmp = run_once()->*[&]() {
// Ensure this is only done once !
static bool done = false;
EXPECT(!done);
done = true;
auto ltmp = ctx.logical_data(lres.shape());
ctx.parallel_for(ltmp.shape(), ltmp.write())->*[] __device__(size_t i, auto tmp) {
tmp(i) = i;
};
return ltmp;
};
auto ltmp2 = run_once(size_t(k % 4))->*[&](size_t val) {
// fprintf(stderr, "COMPUTE FOR %ld\n", val);
auto ltmp = ctx.logical_data(lres.shape());
ctx.parallel_for(ltmp.shape(), ltmp.write())->*[val] __device__(size_t i, auto tmp) {
tmp(i) = val;
};
return ltmp;
};
ctx.parallel_for(lres.shape(), ltmp.read(), lres.rw())->*[] __device__(size_t i, auto tmp, auto res) {
res(i) += tmp(i);
};
}
ctx.finalize();
for (int i = 0; i < N; i++)
{
EXPECT(A[i] == (2 * i + 1) + 2 * i * niter);
}
}